<?php

namespace App\Modules\Menus\Http\Controllers;

use Illuminate\Http\Request;

use Illuminate\Support\Facades\Validator;

use App\Http\Requests;

use Illuminate\Validation\ValidationException;

use App\Http\Controllers\Controller;

use App\Modules\Menus\Models\Menu;

use Carbon\Carbon;

class MenuController extends Controller
{
    public function getAll(){

        return Menu::where('status', '1')->where('is_home', '1')->orderBy('ordering', 'ASC')->get();

    }
    public function getRecords($id='', Request $request)
    {
        //return $request->all();
        try{
            if($id > 0){

                return Menu::find($id);
    
            }else if($request->has('query_string')){
    
                return Menu::where('title', 'LIKE', '%' . $request->query_string . '%')->paginate(10);
             }else{
    
                return Menu::where('status', '!=', '0')->orderBy('id', 'DESC')->paginate(10);
             }
        }catch(\Illuminate\Database\QueryException $ex){
            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);
          }
        
    }

    public function create(Request $request)
    {
        //creating a validator
        $validator = Validator::make($request->all(), [
            'menu_name' => 'required',
            'menu_alias' => 'required',
            'menu_icon' => 'required',
            'menu_ishome' => 'required',
            'menu_status' => 'required',
        ]);

        //if validation fails
        if ($validator->fails()) {
            return array(
                'error' => true,
                'message' => $validator->errors()->all()
            );
        }
    try{
        //creating a new menu
        $menu = new Menu();
        //return $request->all();
        //adding values to the menu table
        $menu->title = $request->menu_name;
        $menu->alias = $request->menu_alias;
        $menu->parent_id = $request->menu_category;
        $menu->view = $request->menu_view;
        $menu->icon = $request->menu_icon;
        $menu->is_home = $request->menu_ishome;
        $menu->ordering = $request->menu_ordering;
        $menu->status = $request->menu_status;
        //saving the menu to database
         if($request->id > 0){
            try{
            $item = $menu->findOrFail($request->id);
            if($item){
                $modification_date =  Carbon::now('Asia/Kolkata');

                 $menu->where('id', $request->id)->update(['title'=>$menu->title,'alias'=>$menu->alias, 'view'=>$menu->view,'icon'=>$menu->icon, 'is_home'=>$menu->is_home, 'ordering'=>$menu->ordering, 'status'=>$menu->status, 'modification_date'=>$modification_date]);

                $res['error'] = false;
                $res['actions'] = 'update';
                return response($res, 200);
            }

            }catch(\Illuminate\Database\QueryException $ex){
                $res['status'] = false;
                $res['message'] = $ex->getMessage();
                return response($res, 500);
              }
        }else{
            $menu->save();

            //returning the added menu
            $res['error'] = false;
            $res['actions'] = 'add';
            return response($res, 200);
        }

    }catch(\Illuminate\Database\QueryException $ex){
            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);
          }
    }

public function getParentCategory($id=''){
    try{
            if($id > 0){
                return Menu::find($id);
            }else{
                return Menu::where('parent_id', '=', 0)->where('status', '1')->orderBy('id', 'DESC')->get();
            } 
     }catch(\Illuminate\Database\QueryException $ex){

        $res['status'] = false;
        $res['message'] = $ex->getMessage();
        return response($res, 500);

     }
    }

public function delete(Request $request)
    {
        $menu = Menu::findOrFail($request->menu_id);
    try{
        if($menu){
           $menu->where('id', $request->menu_id)->update(['status'=>'0']);
            $res['error'] = false;
            return response($res, 200);
        }else{
            $res['error'] = true;
            return response($res, 401);
        }
      }catch(\Illuminate\Database\QueryException $ex){
        $res['status'] = false;
        $res['message'] = $ex->getMessage();
        return response($res, 500);
      }
    }
}
