<?php

require_once __DIR__.'/../vendor/autoload.php';

try {
    (new Dotenv\Dotenv(__DIR__.'/../'))->load();
} catch (Dotenv\Exception\InvalidPathException $e) {
    //
}

/*
|--------------------------------------------------------------------------
| Create The Application
|--------------------------------------------------------------------------
|
| Here we will load the environment and create the application instance
| that serves as the central piece of this framework. We'll use this
| application as an "IoC" container and router for this framework.
|
*/

$app = new Laravel\Lumen\Application(
    realpath(__DIR__.'/../')
);
$app->configureMonologUsing(function (\Monolog\Logger $monolog){
    $handler = new \Monolog\Handler\StreamHandler(storage_path('logs/lumen.log'));
    $handler->setFormatter(new \Monolog\Formatter\LineFormatter(null, null, true, true));
    $monolog->pushHandler($handler);
    return $monolog;
});

 $app->configure('audit');
 $app->configure('mail'); // this is to configure the mail
 $app->alias('mailer', Illuminate\Mail\Mailer::class);
 $app->alias('mailer', Illuminate\Contracts\Mail\Mailer::class);
 $app->alias('mailer', Illuminate\Contracts\Mail\MailQueue::class);
 //$app->configure('services'); // and to configure the services
 $app->withFacades();
 $app->withFacades(true, [Expstudio\Modules\Facades\Module::class => 'Module']);
 $app->withEloquent();

/*
|--------------------------------------------------------------------------
| Register Container Bindings
|--------------------------------------------------------------------------
|
| Now we will register a few bindings in the service container. We will
| register the exception handler and the console kernel. You may add
| your own bindings here if you like or you can make another file.
|
*/

$app->singleton(
    Illuminate\Contracts\Debug\ExceptionHandler::class,
    App\Exceptions\Handler::class
);

$app->singleton(
    Illuminate\Contracts\Console\Kernel::class,
    App\Console\Kernel::class
);

/*
|--------------------------------------------------------------------------
| Register Middleware
|--------------------------------------------------------------------------
|
| Next, we will register the middleware with the application. These can
| be global middleware that run before and after each request into a
| route or middleware that'll be assigned to some specific routes.
|
*/

 //$app->middleware([
    //App\Http\Middleware\Authenticate::class
 //]);

 $app->routeMiddleware([
	'auth' => App\Http\Middleware\Authenticate::class,
	'cors' =>  App\Http\Middleware\CorsMiddleware::class
 ]);

/*
|--------------------------------------------------------------------------
| Register Service Providers
|--------------------------------------------------------------------------
|
| Here we will register all of the application's service providers which
| are used to bind services into the container. Service providers are
| totally optional, so you are not required to uncomment this line.
|
*/

   $app->register(Illuminate\Mail\MailServiceProvider::class);
   $app->register(App\Providers\AuthServiceProvider::class);
   $app->register(Expstudio\Modules\ModulesServiceProvider::class);
   $app->register(App\Providers\EventServiceProvider::class);
   $app->register(OwenIt\Auditing\AuditingServiceProvider::class);
   $app->register(Flipbox\LumenGenerator\LumenGeneratorServiceProvider::class);
   $app->register(Mtownsend\MakeTrait\Providers\MakeTraitServiceProvider::class);

/*
|--------------------------------------------------------------------------
| Load The Application Routes
|--------------------------------------------------------------------------
|
| Next we will include the routes file so that they can all be added to
| the application. This will provide all of the URLs the application
| can respond to, as well as the controllers that may handle them.
|
*/

$app->group(['namespace' => 'App\Http\Controllers'], function ($app) {
    require __DIR__.'/../routes/web.php';
});

return $app;
