<?php
namespace App\Modules\Branch\Http\Controllers;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Http\Requests;
use Illuminate\Validation\ValidationException;
use App\Http\Controllers\Controller;
use App\Modules\Branch\Models\Branch;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
class BranchController extends Controller
{
   /* This fuction is used for get  branch list, and seach for a branch */
    public function getRecords($id='', Request $request)
    {
    try{
            if($id > 0){
                return Branch::find($id);

            }else if($request->has('query_string')){

                return Branch::whereIn('status', ['1','2'])->where('title', 'LIKE', '%' . $request->query_string . '%')->orderBy('title', 'ASC')->paginate(10);

            }else if($request->has('activerecord')){

                return Branch::where('status', '=', $request->activerecord)->orderBy('title', 'ASC')->get();
            }else{

                return Branch::where('status', '!=', '0')->orderBy('title', 'ASC')->paginate(10);
            }
    }catch(\Illuminate\Database\QueryException $ex){

                    $res['status'] = false;
                    $res['message'] = $ex->getMessage();
                    return response($res, 500);

        }
    }
   /* This fuction is used for insert/update branch data */
    public function create(Request $request)
    {
		$id = 0;
         if($request->id > 0 ){
            $id = $request->id;
         }
        //creating a validatorbranches
        $validator = Validator::make($request->all(), [
            'title' => [
						'required',
						Rule::unique('branches')->where(function ($query) use($id) {
							$query->where('status','!=','0')->where('id','!=',$id);
						})
					],
			'branch_code' => ['required','min:1','max:5',
                Rule::unique('branches')->where(function ($query) use($id) {
                    $query->where('status','!=','0')->where('id','!=',$id);
                })
            ]

        ]);
        //if validation fails
        if ($validator->fails()){
			return array(
                'error' => true,
                'message' => $validator->errors()
            );
        }
        try{
        //creating a new branch
        $branch = new Branch();
        $branch->title = $request->title;
        $branch->branch_code = $request->branch_code;
        $branch->fullname = $request->branch_full_name;
        $branch->status = $request->branch_status;
        //updating the branch to database
        if($request->id > 0){
            try{
                $item = $branch->findOrFail($request->id);
                if($item){
                    $modification_date =  Carbon::now('Asia/Kolkata');
                    $branch->where('id', $request->id)->update(['title'=>$branch->title,'branch_code'=>$branch->branch_code, 'fullname'=>$branch->fullname,'status'=>$branch->status,'modification_date'=>$modification_date]);
                    $res['error'] = false;
                    $res['actions'] = 'update';
                    return response($res, 200);
                }
            }catch(\Illuminate\Database\QueryException $ex){

                    $res['status'] = false;
                    $res['message'] = $ex->getMessage();
                    return response($res, 500);

            }
        }else{
        //saving the branch to database
            $branch->save();
            $res['error'] = false;
            $res['actions'] = 'add';
            return response($res, 200);
         }
        }catch (\Illuminate\Database\QueryException $ex) {
            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);
        }
    }


    public function delete(Request $request)
    {
        $branch = Branch::findOrFail($request->branch_id);
        try{
            if($branch){

                    $branch->where('id', $request->branch_id)->update(['status'=>'0']);
                     $res['error'] = false;
                     return response($res, 200);
            }else{
                    $res['error'] = true;
                     return response($res, 401);
            }
        }catch(\Illuminate\Database\QueryException $ex){
            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);
          }
    }
}
