<?php
namespace App\Modules\Instruction\Http\Controllers;
use Illuminate\Http\Request;
use App\Http\Requests;
use App\Http\Controllers\Controller;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Facades\Validator;
use App\Modules\Instruction\Models\Instruction;
use App\Modules\Billing\Models\Prescribemedicine;
use App\Modules\Billing\Models\Billing;
use Carbon\Carbon;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use Illuminate\Support\Facades\DB;
use App\Modules\Discount\Models\discount;
use App\Modules\Billing\Models\Payment;
use App\Modules\Billing\Models\PatientBillingCharge;
use App\Modules\Billing\Models\visitPaymentModes;
use App\Modules\Billing\Models\billingPatientCharge;
use App\Modules\Branch\Models\Branch;
use App\Modules\User\Models\DoctorCharges;//This model for doctor charges
//This below imported Libarary is used for HTTP request for other server
use GuzzleHttp\Client; 

class InstructionController extends Controller
{
    public function create($id='',Request $request){
        $validator = Validator::make($request->all(), [
            'instruction_name' => 'required',
            'status' => 'required'
        ]);
        //if validation fails
        if ($validator->fails()) {
            return array(
                'error' => true,
                'message' => $validator->errors()
            );
        }
 try{
        $instruction = new Instruction();
        $instruction->instruction_name = $request->instruction_name;
        $instruction->status = $request->status;
        //this below conditional block is used for update records in database 
     if((isset($request->id)) && ($request->id > 0)){
             try{
                    //for Medicine data
                    $item = $instruction->findOrFail($request->id);
                        if($item){
                            $modification_date =  Carbon::now('Asia/Kolkata');
                            $instruction->where('id', $request->id)->update(['instruction_name'=>$instruction->instruction_name, 'status'=>$instruction->status, 'modification_date'=>$modification_date]);
                            $res['error'] = false;
                            $res['actions'] = 'update';
                            return response($res, 200);
                        }
                }catch(\Illuminate\Database\QueryException $ex){
                    $res['status'] = false;
                    $res['message'] = $ex->getMessage();
                    return response($res, 500);
                } 	
            }else{
                    $instruction->save();
                    $res['error'] = false;
                    $res['actions'] = 'add';
                    return response($res, 200);
                      
            }
    
        }catch(\Illuminate\Database\QueryException $ex){
            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);
        } 	
    }

    public function getRecords($id='',Request $request){
            /*This below condition is used for distinguish edit form request, In this case we get details of perticular instruction */
            if($id > 0){
                
                return Instruction::find($id);
            }
             /*This below condition is used for distinguish getting all data list form request, */
            if($request->has('datalist')){

               // return Instruction::where('status','=', ['1','3'])->orderBy('id', 'DESC')->get();
			   return Instruction::whereNotIn('status', ['0','2'])->orderBy('id', 'DESC')->get();

            }
           // return Instruction::where('status','=', ['1','2'])->orderBy('id', 'DESC')->get();
		   return Instruction::where('status', '!=', '0')->orderBy('id', 'DESC')->get();
    }

    //  Delete Record for particular instruction id
        public function delete(Request $request)
        {
            $instruction = Instruction::findOrFail($request->instruction_id);
            try{
                if($instruction){

                        $instruction->where('id', $request->instruction_id)->update(['status'=>'0']);
                        $res['error'] = false;
                        return response($res, 200);
                }else{
                        $res['error'] = true;
                        return response($res, 401);
                }
            }catch(\Illuminate\Database\QueryException $ex){
                $res['status'] = false;
                $res['message'] = $ex->getMessage();
                return response($res, 500);
            }
        }
		public function valueSign($num)
			{
			  return $sign = $num < 0 ? -1 : ( $num > 0 ? 1 : 0 );
			  //or
			 // return $sign = $num ? $num / abs($num) : 0;
			}
		public function exportHistory(Request $request){
			$billing_array = array();
			$medicine_total = array();
			$discounts = array();
			$main_discounts = array();
			$main_total = array();
			$charge_array = array();
			$mode_array = array();
			$main_mode_array = array();
			$doctor_charge_array = array();
			$amt= array();
			$mde = array();
			$rmmde = array();
			$previous_due_array = array();
			// for repeat medicine
			$medicine_completed_medicine_total = array();
			$medicine_completed_main_discounts = array();
			$medicine_completed_charge_array = array();
			$medicine_completed_main_total= array();
			$medicine_completed_mode_array= array();
			$medicine_completed_discounts_array = array();
			$medicine_completed_discounts_amt= array();
			$medicine_completed_discounts_mde= array();
			$medicine_completed_previous_due_array = array();
			$medicine_completed_previous_bill_paid_array = array();
			//this below code line is used for getting core existing APIs url
				//$Core_Api_root = env('CORE_API_ROOT', true);
				$Core_Api_root = "http://clinic.drkbanerjee.com:4300/";
				//print_r($Core_Api_root);exit;
				$client = new Client();
				$branch = $request->branch_id;
				$shift = $request->shift_id;
				$date = $request->export_date;
				$new_date = date_create($date);
				$visit_date= date_format($new_date, "d-M-Y");
				$file_date= date_format($new_date, "d-M-Y");
				$branch_name= Branch::where('id', $branch)->first();
				$file_name = $file_date.'_'.$branch_name->title;
				/***************** code for patient*****************/
				$res = $client->request('GET', $Core_Api_root.'appointments/branch',[
					'query' => ['branch_id' => $branch, 'dateTime'=>$date, 'shift_id'=> $shift]
				]);
				
				
				//print_r($res);exit;
				if ($res->getStatusCode() == 200) { // 200 OK
					$result = $res->getBody()->getContents();
				}
				$result = json_decode($result);
				
				/******************doctorsAssigned*************************/
				$res_assigned  = $client->request('GET', $Core_Api_root.'appointments/doctorsAssignedExport',[
					'query' => ['branch_id' => $branch, 'dateTime'=>$date, 'shift_id'=> $shift]
				]);
				//print_r($res);exit;
				if ($res_assigned ->getStatusCode() == 200) { // 200 OK
					$result_assigned  = $res_assigned ->getBody()->getContents();
				}
				$result_assigned = json_decode($result_assigned);
				
				/****************************completed********************************/
				
				/* $res_medicine_completed = $client->request('GET', $Core_Api_root.'appointments/exportmedicinecompleted',[
					'query' => ['branch_id' => $branch, 'dateTime'=>$date, 'shift_id'=> $shift]
				]);
				//print_r($res);exit;
				if ($res_medicine_completed->getStatusCode() == 200) { // 200 OK
					$result_medicine_completed = $res_medicine_completed->getBody()->getContents();
				}
				$result_medicine_completed = json_decode($result_medicine_completed);
				
				// for medicine total 
				
				foreach($result_medicine_completed->data as $rs_completed){
					//print_r($rs_completed->visit_id);exit;
				if(!empty($rs_completed->appointment_id) && !empty($rs_completed->billing_id)){
						$total=0;
						$precribe_medicine_list = DB::table('billing as b')->LeftJoin('prescribemedicine as pm', 'pm.billing_id', '=', 'b.id')->where('b.status', '=', '1')->where('pm.parent_id', '=', '0')->where('pm.appointment_id', $rs_completed->appointment_id)->where('pm.billing_id', $rs_completed->billing_id)->get();
									foreach($precribe_medicine_list as $key => $list){
									$mix_medicine_list = Prescribemedicine::where('parent_id',$list->id)->get();
									$mix_medicine_test = Prescribemedicine::where('parent_id',$list->id)->get()->max('medicine_amount');
									if($mix_medicine_test > 0){
										$total+=$mix_medicine_test;
									}else{
										$total+=$list->medicine_amount;
									}

						}
						$total_c = $total;
						$total_price = number_format((float)$total_c, 2, '.', '');
						array_push($medicine_completed_medicine_total,  $total_price);
					}
				}

			// For Visit discount and wave consultancy_fees
			 foreach($result_medicine_completed->data as $medicine_completed)
					{
						if(!empty($medicine_completed->appointment_id) && !empty($medicine_completed->billing_id)){
							$patient_discount = discount::select('discount_on_med','discount_reason','discount_after_med_cost','id',DB::raw("(SELECT Name FROM cl_users WHERE user_id = cl_patient_discount.created_by) as discount_apply_name"))->where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->get()->first();
							if(!empty($patient_discount)){
								
								$medicine_completed_discounts_array['discount_on_med'] = $patient_discount->discount_on_med;
								$medicine_completed_discounts_array['discount_id'] = $patient_discount->id;
								$medicine_completed_discounts_array['discount_after_med_cost'] = $patient_discount->discount_after_med_cost;
								$medicine_completed_discounts_array['med_resson'] = $patient_discount->discount_reason;
								$medicine_completed_discounts_array['discount_apply_name'] = $patient_discount->discount_apply_name;
								
							}
							else{
								$medicine_completed_discounts_array['discount_on_med'] = 0;
								$medicine_completed_discounts_array['discount_after_med_cost'] = 0;
							}
							$waive_discount =  Billing::select('waive_consultancy_fees','waive_percentage','waive_type','consultancy_waived_reason', DB::raw("(SELECT Name FROM cl_users WHERE user_id = cl_billing.consultancy_waived_by) as consultacy_apply_name"))->where('appointment_id', $medicine_completed->appointment_id)->where('id', $medicine_completed->billing_id)->get()->first();
							$medicine_completed_discounts_array['waive_consultancy_fees'] = $waive_discount->waive_consultancy_fees;
							$medicine_completed_discounts_array['waive_type'] = $waive_discount->waive_type;
							$medicine_completed_discounts_array['consultacy_reason'] = $waive_discount->consultancy_waived_reason;
							$medicine_completed_discounts_array['consultacy_apply_name'] = $waive_discount->consultacy_apply_name;
							array_push($medicine_completed_main_discounts,  $medicine_completed_discounts_array);
						}
						
					} 
			
					// for visit due and received amount
					foreach($result_medicine_completed->data as $medicine_completed){
					if(!empty($medicine_completed->appointment_id) && !empty($medicine_completed->billing_id)){
						
							$count_row = Payment::where('appointment_id', $medicine_completed->appointment_id)->get()->count();
								$recieved_amount= Payment::select('total_due_amount', 'recieved_amount', 'visit_due_amount', 'credit_amount', 'debit_amount', 'previous_bill_paid_amount', DB::raw("(SELECT Name FROM cl_users WHERE user_id = cl_payment.created_by) as biller_name"), 'recieved_amount', 'transaction_time', 'refund_flag')->where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->get();
								
								$credit_amount= Payment::where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->where('refund_flag', '0')->sum('recieved_amount');
								$debit_amount= Payment::where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->where('refund_flag', '1')->sum('recieved_amount');
								$total_received_amount = abs($credit_amount-$debit_amount);
								if($count_row > 0){
									if(count($recieved_amount)>0){
										$medicine_completed_discounts_amt['total_due_amount'] = $recieved_amount[0]->total_due_amount;
										$medicine_completed_discounts_amt['previous_bill_paid_amount'] = $recieved_amount[0]->previous_bill_paid_amount;
										$medicine_completed_discounts_amt['visit_due_amount'] = $recieved_amount[0]->visit_due_amount;
									}else{
										$medicine_completed_discounts_amt['total_due_amount'] =0;
										$medicine_completed_discounts_amt['visit_due_amount'] =0;
										$medicine_completed_discounts_amt['previous_bill_paid_amount'] =0;
										
									}
									$medicine_completed_discounts_amt['recieved_amount'] = number_format((float)$total_received_amount, 2, '.', '');
								}else{
									$medicine_completed_discounts_amt['total_due_amount'] =0;
									$medicine_completed_discounts_amt['previous_bill_paid_amount'] =0;
									$medicine_completed_discounts_amt['visit_due_amount'] =0;
									$medicine_completed_discounts_amt['recieved_amount'] = 0;
								}
								array_push($medicine_completed_main_total,  $medicine_completed_discounts_amt);
						}
					} 
					
			// for another charge
				foreach($result_medicine_completed->data as $medicine_completed){
						if(!empty($medicine_completed->appointment_id) && !empty($medicine_completed->billing_id)){
							 $charge = billingPatientCharge::where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->sum('charge_amount');
							 array_push($medicine_completed_charge_array,  $charge);
						}
					}
				
				// for payment mode
				
				 foreach($result_medicine_completed->data as $medicine_completed){
					if(!empty($medicine_completed->appointment_id) && !empty($medicine_completed->billing_id)){

						$getPaymentIdList= Payment::select('id')->where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->orderBy('transaction_time', 'ASC')->get();
						
						$rm_p_mode_array = array();
						
						 foreach($getPaymentIdList as  $payid){
							
							$rm_payment_mode_list = visitPaymentModes::select('mode_price','reference_no', DB::raw("(SELECT name FROM cl_payment_mode WHERE id = mode_id) as transaction_mode"))->whereIn('payment_id', [$payid->id])->get();
							
							array_push($rm_p_mode_array,  $rm_payment_mode_list);
						} 
						
						array_push($medicine_completed_mode_array, $rm_p_mode_array);
						
					}
				}	
				// for previous due amount
				foreach($result_medicine_completed->data as $medicine_completed){
						if(!empty($medicine_completed->billing_id) && !empty($medicine_completed->appointment_id)){
						 $complete_medicine_pending_array =array();
						 $pendind_list =  DB::table('payment')->select('transaction_status','debit_amount','credit_amount','due_amount','transaction_time')->where('patient_id', $medicine_completed->p_id)->where('status', '1')->orderBy('id', 'DESC')->where('appointment_id','!=', $medicine_completed->appointment_id)->take(1)->first();
						 if(!empty($pendind_list)){
							$balance = ($pendind_list->debit_amount-$pendind_list->credit_amount);
							$complete_medicine_pending_array['due_amount'] =number_format((float)$balance, 2, '.', '');
							$complete_medicine_pending_array['transaction_status'] = $pendind_list->transaction_status;
							
						 }else{
							 $complete_medicine_pending_array['due_amount']=0;
							 $complete_medicine_pending_array['transaction_status']='';
						 }
						 array_push($medicine_completed_previous_due_array, $complete_medicine_pending_array);
					 }
				} */
				/*****************************medicinecompleted********************************/
				
				$res_completed = $client->request('GET', $Core_Api_root.'appointments/exportcompleted',[
					'query' => ['branch_id' => $branch, 'dateTime'=>$date, 'shift_id'=> $shift]
				]);
				//print_r($res);exit;
				if ($res_completed->getStatusCode() == 200) { // 200 OK
					$result_completed = $res_completed->getBody()->getContents();
				}
				$result_completed = json_decode($result_completed);	
				/* echo '<pre>';
				print_r($result_completed);
				echo '</pre>';
				exit;			 */
				// for medicine total 
				 foreach($result_completed->data as $medicine_completed){
				if(!empty($medicine_completed->billing_id) && !empty($medicine_completed->appointment_id)){
							$total=0;
							$precribe_medicine_list = DB::table('billing as b')->select('pm.id','pm.medicine_amount')->LeftJoin('prescribemedicine as pm', 'pm.billing_id', '=', 'b.id')->where('b.status', '=', '1')->where('pm.parent_id', '=', '0')->where('pm.appointment_id', $medicine_completed->appointment_id)->where('pm.billing_id', $medicine_completed->billing_id)->get();
										foreach($precribe_medicine_list as $key => $list){
										//$mix_medicine_list = Prescribemedicine::where('parent_id',$list->id)->get();
										$mix_medicine_test = Prescribemedicine::where('parent_id',$list->id)->get()->max('medicine_amount');
										if($mix_medicine_test > 0){
											$total+=$mix_medicine_test;
										}else{
											$total+=$list->medicine_amount;
										}

							}
							$total_c = $total;
							$total_price = number_format((float)$total_c, 2, '.', '');
							array_push($medicine_total,  $total_price);
					}
				}
				
			// For Visit discount and wave consultancy_fees
			 foreach($result_completed->data as $medicine_completed)
					{
						if(!empty($medicine_completed->appointment_id) && !empty($medicine_completed->billing_id)){
							$patient_discount = discount::select('discount_on_med','discount_reason','discount_after_med_cost','id',DB::raw("(SELECT Name FROM cl_users WHERE user_id = cl_patient_discount.created_by) as discount_apply_name"))->where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->get()->first();
							if(!empty($patient_discount)){
								
								$discounts['discount_on_med'] = $patient_discount->discount_on_med;
								$discounts['discount_id'] = $patient_discount->id;
								$discounts['discount_after_med_cost'] = $patient_discount->discount_after_med_cost;
								$discounts['med_resson'] = $patient_discount->discount_reason;
								$discounts['discount_apply_name'] = $patient_discount->discount_apply_name;
								
							}
							else{
								$discounts['discount_on_med'] = 0;
								$discounts['discount_after_med_cost'] = 0;
							}

							$waive_discount =  Billing::select('waive_consultancy_fees','waive_percentage','waive_type','consultancy_waived_reason', DB::raw("(SELECT Name FROM cl_users WHERE user_id = cl_billing.consultancy_waived_by) as consultacy_apply_name"))->where('appointment_id', $medicine_completed->appointment_id)->where('id', $medicine_completed->billing_id)->get()->first();
							$discounts['waive_consultancy_fees'] = $waive_discount->waive_consultancy_fees;
							$discounts['waive_type'] = $waive_discount->waive_type;
							$discounts['waive_percentage'] = $waive_discount->waive_percentage;
							//$discounts['consultacy_apply_name'] = $waive_discount->consultacy_apply_name;
							array_push($main_discounts,  $discounts);
						}
						
					} 
				

				// for visit due and received amount
				foreach($result_completed->data as $medicine_completed){
					if(!empty($medicine_completed->appointment_id) && !empty($medicine_completed->billing_id)){
							$count_row = Payment::where('appointment_id', $medicine_completed->appointment_id)->get()->count();
								$recieved_amount= Payment::select('total_due_amount', 'recieved_amount', 'visit_due_amount', 'credit_amount', 'debit_amount', 'previous_bill_paid_amount', DB::raw("(SELECT Name FROM cl_users WHERE user_id = cl_payment.created_by) as biller_name"), 'recieved_amount', 'transaction_time', 'refund_flag')->where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->get();
								
								$credit_amount= Payment::where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->where('refund_flag', '0')->sum('recieved_amount');
								$debit_amount= Payment::where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->where('refund_flag', '1')->sum('recieved_amount');
								
								$total_received_amount = abs($credit_amount-$debit_amount);
								if($count_row > 0){
									if(count($recieved_amount)>0){
										$amt['total_due_amount'] = $recieved_amount[0]->total_due_amount;
										$amt['previous_bill_paid_amount'] = $recieved_amount[0]->previous_bill_paid_amount;
										$amt['visit_due_amount'] = $recieved_amount[0]->visit_due_amount;
									}else{
										$amt['total_due_amount'] =0;
										$amt['visit_due_amount'] =0;
										$amt['previous_bill_paid_amount'] =0;
										
									}
									$amt['recieved_amount'] = number_format((float)$total_received_amount, 2, '.', '');
								}else{
									$amt['total_due_amount'] =0;
									$amt['visit_due_amount'] =0;
									$amt['previous_bill_paid_amount'] =0;
									$amt['recieved_amount'] = 0;
								}
								array_push($main_total,  $amt);
						}
					} 
					
				/* echo '<pre>';
				print_r($main_total);
				echo '</pre>';
				exit; */
				// for another charge
				 foreach($result_completed->data as $medicine_completed){
					if(!empty($medicine_completed->appointment_id) && !empty($medicine_completed->billing_id)){
						 $charge = billingPatientCharge::where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->sum('charge_amount');
						 array_push($charge_array,  $charge);
					}
				} 
				
				// For consultancy_fees
				 foreach($result_completed->data as $medicine_completed)
					{
							$doctor_chrg = DoctorCharges::select('price')->where('doctor_id', $medicine_completed->doctor_id)->where('charge_code', 'LIKE', "%cns%")->get();
							if(count($doctor_chrg)>0){
								array_push($doctor_charge_array,  $doctor_chrg[0]->price);
							}else{
								array_push($doctor_charge_array,  0);
							}
							
						
					} 
					
				// for payment mode
				 foreach($result_completed->data as $medicine_completed){
					if(!empty($medicine_completed->appointment_id) && !empty($medicine_completed->billing_id)){

						$getPaymentIdList= Payment::select('id')->where('appointment_id', $medicine_completed->appointment_id)->where('billing_id', $medicine_completed->billing_id)->orderBy('transaction_time', 'ASC')->get();
						
						$p_mode_array = array();
						
						 foreach($getPaymentIdList as  $payid){
							
							$payment_mode_list = visitPaymentModes::select('mode_price','reference_no', DB::raw("(SELECT name FROM cl_payment_mode WHERE id = mode_id) as transaction_mode"))->whereIn('payment_id', [$payid->id])->get();
							
							array_push($p_mode_array,  $payment_mode_list);
						} 
						
						array_push($mode_array, $p_mode_array);
						
					}
				} 
			// for previous due amount
				foreach($result_completed->data as $medicine_completed){
						if(!empty($medicine_completed->billing_id) && !empty($medicine_completed->appointment_id)){
						 $pending_array =array();
						 $pendind_list =  DB::table('payment')->select('transaction_status','debit_amount','credit_amount','due_amount','transaction_time')->where('patient_id', $medicine_completed->p_id)->where('status', '1')->orderBy('id', 'DESC')->where('appointment_id','!=', $medicine_completed->appointment_id)->take(1)->first();
						 if(!empty($pendind_list)){
							$balance = ($pendind_list->debit_amount-$pendind_list->credit_amount);
							$pending_array['due_amount'] =number_format((float)$balance, 2, '.', '');
							$pending_array['transaction_status'] = $pendind_list->transaction_status;
							
						 }else{
							$pending_array['due_amount'] =0;
							$pending_array['transaction_status']='';
						 }
						 
						 array_push($previous_due_array, $pending_array);
					 }
				}
			        /* echo '<pre>';
					print_r($result_completed);
					print_r($medicine_total);
					print_r($main_discounts);
					print_r($charge_array);
					print_r($doctor_charge_array);
					print_r($mode_array);
					print_r($previous_due_array);
					echo '</pre>';  
					exit;   */
				/*************************************************************************/
				//echo "Atleast function call";exit;
				$objPHPExcel = new Spreadsheet();
				$objPHPExcel->setActiveSheetIndex(0);
				$i = 1;
				$objPHPExcel->getActiveSheet()->SetCellValue('A'.$i, 'S.No.');
				$objPHPExcel->getActiveSheet()->SetCellValue('B'.$i, 'Patient ID');
				$objPHPExcel->getActiveSheet()->SetCellValue('C'.$i, 'Patient Name');
				$objPHPExcel->getActiveSheet()->SetCellValue('D'.$i, 'Doctor');
				$objPHPExcel->getActiveSheet()->SetCellValue('E'.$i, 'Appointment Time');

				foreach($result->booked as $rs){
					$i++;
					if($rs->OldRegistrationNo){
						$registerationNo = $rs->OldRegistrationNo;
					} else{
						$registerationNo =  'NP';
					}
					//ucwords($rs->title).' '.
					$objPHPExcel->getActiveSheet()->SetCellValue('A'.$i, (int)($i - 1));
					$objPHPExcel->getActiveSheet()->SetCellValue('B'.$i, $registerationNo);
					$objPHPExcel->getActiveSheet()->SetCellValue('C'.$i, ucwords($rs->first_name).' '.ucwords($rs->last_name));
					$objPHPExcel->getActiveSheet()->SetCellValue('D'.$i, $rs->label);
					$objPHPExcel->getActiveSheet()->SetCellValue('E'.$i, $rs->appointment_time);
				}

				$objPHPExcel->getActiveSheet()->setTitle("Booked (Not Assigned)");

				$objWorkSheet = $objPHPExcel->createSheet(1);

				// $objPHPExcel->setActiveSheetIndex(1);
				$i = 1;

				$objWorkSheet->SetCellValue('A'.$i, 'S.No.');
				$objWorkSheet->SetCellValue('B'.$i, 'Patient ID');
				$objWorkSheet->SetCellValue('C'.$i, 'Patient Name');
				$objWorkSheet->SetCellValue('D'.$i, 'Doctor');
				$objWorkSheet->SetCellValue('E'.$i, 'Appointment Time');
				
				foreach($result->onspot as $rs){
					// echo "<pre>"; print_r($rs); exit;
					if($rs->medicine_queue_number == null){
						$i++;
						if($rs->OldRegistrationNo){
							$registerationNo = $rs->OldRegistrationNo;
						} else{
							$registerationNo =  'NP';
						}
						
						//ucwords($rs->title).' '.
						$objWorkSheet->SetCellValue('A'.$i, (int)($i - 1));
						$objWorkSheet->SetCellValue('B'.$i, $registerationNo);
						$objWorkSheet->SetCellValue('C'.$i, ucwords($rs->first_name).' '.ucwords($rs->last_name));
						$objWorkSheet->SetCellValue('D'.$i, $rs->label);
						$objWorkSheet->SetCellValue('E'.$i, $rs->appointment_time);
					}
				}
				$objWorkSheet->setTitle('On Spot (Not Assigned)');
				$objWorkSheet_completed = $objPHPExcel->createSheet(2);
				$i = 1;
				$objWorkSheet_completed->SetCellValue('A'.$i, 'S.No.');
				$objWorkSheet_completed->SetCellValue('B'.$i, 'Patient ID');
				$objWorkSheet_completed->SetCellValue('C'.$i, 'Patient Name');
				$objWorkSheet_completed->SetCellValue('D'.$i, 'Doctor');
				$objWorkSheet_completed->SetCellValue('E'.$i, 'Date');
				$objWorkSheet_completed->SetCellValue('F'.$i, 'Appointment Time');
				$objWorkSheet_completed->SetCellValue('G'.$i, 'Shift');
				$objWorkSheet_completed->SetCellValue('H'.$i, 'Location');
				$objWorkSheet_completed->SetCellValue('I'.$i, 'Consultancy Fees');
				$objWorkSheet_completed->SetCellValue('J'.$i, 'Consultancy waiver');
				$objWorkSheet_completed->SetCellValue('K'.$i, 'Medicine Price');
				$objWorkSheet_completed->SetCellValue('L'.$i, 'Medicine Discounted Price');
				$objWorkSheet_completed->SetCellValue('M'.$i, 'Medicine Discount');
				$objWorkSheet_completed->SetCellValue('N'.$i, 'Previous Due (+)/Advance (-)');
				$objWorkSheet_completed->SetCellValue('O'.$i, 'Other Charge Total');
				$objWorkSheet_completed->SetCellValue('P'.$i, 'Previous Paid Amount ');
				$objWorkSheet_completed->SetCellValue('Q'.$i, 'Visit Due');
				$objWorkSheet_completed->SetCellValue('R'.$i, 'Amount Received');
				$objWorkSheet_completed->SetCellValue('S'.$i, 'Current Due (+)/Advance (-)');
				$objWorkSheet_completed->SetCellValue('T'.$i, 'Amount Refunded');
				$objWorkSheet_completed->SetCellValue('U'.$i, 'Mode of Payment');
				$objWorkSheet_completed->getStyle('A1:U1')->getFont()->setBold(true);
				$objWorkSheet_completed->getStyle('A1:U1')->getFont()->setSize(16);
				$objWorkSheet_completed->getStyle('M1')->getNumberFormat()->setFormatCode("0%");
				$objWorkSheet_completed->getStyle('N1')->getNumberFormat()->setFormatCode("+0;-0;0");
				$objWorkSheet_completed->freezePane('A2');
				
				foreach($result_completed->data as $key=>$rs_completed){
					// echo "<pre>"; print_r($rs); exit;
					if($rs_completed->medicine_complete!=1 && $rs_completed->doctor_id!=0){
					$i++;
					if($rs_completed->OldRegistrationNo){
						$registerationNo = $rs_completed->OldRegistrationNo;
					} else{
						$registerationNo =  'NP';
					}
					if($rs_completed->shift_id==1){
						$login_shift = 'M';
					} else{
						$login_shift =  'E';
					}
					$total_due_amount = $main_total[$key]['total_due_amount'];
					$recieved_amount = $main_total[$key]['recieved_amount'];
					$previous_bill_paid_amount = $main_total[$key]['previous_bill_paid_amount'];
					//$total_due1 = $total_due_amount - $recieved_amount;
					 if($recieved_amount==$total_due_amount){
                                $remaining_balance_status = '';
                                    $remain_balance = 0.00;
                               }else if($recieved_amount< $total_due_amount){
                                $remaining_balance_status = '+';
                                $balance = $total_due_amount-$recieved_amount;
								$balance_format = number_format((float)$balance, 2, '.', '');
                                $remain_balance = $remaining_balance_status.$balance_format;
                              }else if($recieved_amount > $total_due_amount){
                                $remaining_balance_status = '-';
                                $balance = $recieved_amount-$total_due_amount;
								$balance_format = number_format((float)$balance, 2, '.', '');
                                $remain_balance = $remaining_balance_status.' '.$balance_format;
								
                              } 
							
					 if($main_discounts[$key]['waive_type']=='Waived'){
						 
						$consultancy_waived = $main_discounts[$key]['waive_consultancy_fees'];
						
					}else if($main_discounts[$key]['waive_type']=='' && $main_discounts[$key]['waive_percentage']==100){
						
						$consultancy_waived = $main_discounts[$key]['waive_consultancy_fees'];
						
					}else if($main_discounts[$key]['waive_type']=='Discount Category' && $main_discounts[$key]['waive_percentage']==100){
						
						$consultancy_waived = $main_discounts[$key]['waive_consultancy_fees'];
						
					}else if($main_discounts[$key]['waive_type']=='Not Set' && $main_discounts[$key]['waive_percentage']==0){
						
						$consultancy_waived = 0.00;
					}else{
						$consultancy_waived = 0.00;
					} 
					if(count($mode_array[$key])>0){
							   foreach($mode_array[$key] as $mode_list){
								   $temp_mode='';
								   foreach($mode_list as $mode_name){
									   
									   $temp_mode .=  $mode_name->transaction_mode.'('.$mode_name->mode_price.')'.", ";							   
								   } 
								   $mde['mode_name'] =   rtrim($temp_mode, ", ");
								} 
					}else{
						 $mde['mode_name'] = 'Mark Complete';
						
					}
					if($previous_due_array[$key]['transaction_status']=='Debit'){
						$remaining_balance_status='+';
						$balance = $previous_due_array[$key]['due_amount'];
						$previous_due_balance = $remaining_balance_status.$balance;
					}else if($previous_due_array[$key]['transaction_status']=='Credit'){
						$remaining_balance_status='-';
						$balance = $previous_due_array[$key]['due_amount'];
						$previous_due_balance = $balance;
					}else{
						$previous_due_balance=0.00;
					}
					
					
					//ucwords($rs_completed->title).' '.
					$objWorkSheet_completed->SetCellValue('A'.$i, (int)($i - 1));
					$objWorkSheet_completed->SetCellValue('B'.$i, $registerationNo);
					$objWorkSheet_completed->SetCellValue('C'.$i, ucwords($rs_completed->first_name).' '.ucwords($rs_completed->last_name));
					$objWorkSheet_completed->SetCellValue('D'.$i, $rs_completed->label);
					$objWorkSheet_completed->SetCellValue('E'.$i, $visit_date);
					$objWorkSheet_completed->SetCellValue('F'.$i, $rs_completed->appointment_time);
					$objWorkSheet_completed->SetCellValue('G'.$i, $login_shift);
					$objWorkSheet_completed->SetCellValue('H'.$i, $rs_completed->branch);
					$objWorkSheet_completed->SetCellValue('I'.$i, $doctor_charge_array[$key]);
					$objWorkSheet_completed->SetCellValue('J'.$i, $consultancy_waived);
					$objWorkSheet_completed->SetCellValue('K'.$i, $medicine_total[$key]);
					$objWorkSheet_completed->SetCellValue('L'.$i, $main_discounts[$key]['discount_after_med_cost']);
					$objWorkSheet_completed->SetCellValue('M'.$i, $main_discounts[$key]['discount_on_med']);
					$objWorkSheet_completed->SetCellValue('N'.$i, $previous_due_balance);
					$objWorkSheet_completed->SetCellValue('O'.$i, $charge_array[$key]);
					$objWorkSheet_completed->SetCellValue('P'.$i, $previous_bill_paid_amount);
					
					$objWorkSheet_completed->SetCellValue('Q'.$i, $total_due_amount);
					$objWorkSheet_completed->SetCellValue('R'.$i, $recieved_amount);
					$objWorkSheet_completed->SetCellValue('S'.$i, $remain_balance);
					$objWorkSheet_completed->SetCellValue('T'.$i, 0);
					$objWorkSheet_completed->SetCellValue('U'.$i, $mde['mode_name']);
				  }
				}
				
				$objWorkSheet_completed->setTitle('Completed');

				$j = 1;
				$sheet_number = 3;

				foreach($result_assigned as $rs_assigned){
					$i = 1;
					$objWorkSheet_assigned = $objPHPExcel->createSheet($sheet_number);
					// $objWorkSheet_assigned->SetCellValue(chr(64+$j).$i, $rs_assigned->doctor_name);
					// $i++;
					foreach($rs_assigned->appointments as $doc_appointments){
						if($doc_appointments->OldRegistrationNo){
							$registerationNo = $doc_appointments->OldRegistrationNo;
						} else{
							$registerationNo =  'NP';
						}
						//ucwords($doc_appointments->title).' '.
						$objWorkSheet_assigned->SetCellValue('A'.$i, ucwords($doc_appointments->first_name).' '.ucwords($doc_appointments->middle_name).' '.$doc_appointments->last_name);
						$objWorkSheet_assigned->SetCellValue('B'.$i, $registerationNo);
						if($doc_appointments->assigned_dr_1 == $rs_assigned->id && $doc_appointments->assigned_dr_2 == $rs_assigned->id){
							$objWorkSheet_assigned->SetCellValue('C'.$i, 'Both');
						} else if($doc_appointments->assigned_dr_1 == $rs_assigned->id && $doc_appointments->assigned_dr_2 == ''){
							$objWorkSheet_assigned->SetCellValue('C'.$i, 'Both');
						} else if($doc_appointments->assigned_dr_1 == $rs_assigned->id && $doc_appointments->assigned_dr_2 <> $rs_assigned->id){
							$objWorkSheet_assigned->SetCellValue('C'.$i, 'Case Taken');
						} else if($doc_appointments->assigned_dr_1 <> $rs_assigned->id && $doc_appointments->assigned_dr_2 == $rs_assigned->id){
							$objWorkSheet_assigned->SetCellValue('C'.$i, 'Prescribed');
						}
						$i++;
					}
					// echo chr(64+$j).$i, $rs_assigned->doctor_name; exit;
					$j++;
					$objWorkSheet_assigned->setTitle($rs_assigned->doctor_name);
					$sheet_number++;
				}
				

				/* Medicine Queue */
				$objWorkSheet_medicine_completed = $objPHPExcel->createSheet($sheet_number);
				$i = 1;
				$objWorkSheet_medicine_completed->SetCellValue('A'.$i, 'S.No.');
				$objWorkSheet_medicine_completed->SetCellValue('B'.$i, 'Patient ID');
				$objWorkSheet_medicine_completed->SetCellValue('C'.$i, 'Patient Name');
				$objWorkSheet_medicine_completed->SetCellValue('D'.$i, 'Date');
				$objWorkSheet_medicine_completed->SetCellValue('E'.$i, 'Appointment Time');
				$objWorkSheet_medicine_completed->SetCellValue('F'.$i, 'Shift');
				$objWorkSheet_medicine_completed->SetCellValue('G'.$i, 'Location');
				$objWorkSheet_medicine_completed->SetCellValue('H'.$i, 'Medicine Price');
				$objWorkSheet_medicine_completed->SetCellValue('I'.$i, 'Medicine Discounted Price');
				$objWorkSheet_medicine_completed->SetCellValue('J'.$i, 'Medicine Discount');
				$objWorkSheet_medicine_completed->SetCellValue('K'.$i, 'Previous Due (+)/Advance (-)');
				$objWorkSheet_medicine_completed->SetCellValue('L'.$i, 'Other Charge Total');
				$objWorkSheet_medicine_completed->SetCellValue('M'.$i, 'Previous Paid Amount ');
				$objWorkSheet_medicine_completed->SetCellValue('N'.$i, 'Visit Due');
				$objWorkSheet_medicine_completed->SetCellValue('O'.$i, 'Amount Received');
				$objWorkSheet_medicine_completed->SetCellValue('P'.$i, 'Current Due (+)/Advance (-)');
				$objWorkSheet_medicine_completed->SetCellValue('Q'.$i, 'Amount Refunded');
				$objWorkSheet_medicine_completed->SetCellValue('R'.$i, 'Mode of Payment');
				$objWorkSheet_medicine_completed->getStyle('A1:R1')->getFont()->setBold(true);
				$objWorkSheet_medicine_completed->getStyle('A1:R1')->getFont()->setSize(16);
				$objWorkSheet_medicine_completed->freezePane('A2');
				
				foreach($result_completed->data as $key1=>$rs_medicine_completed){
					// echo "<pre>"; print_r($rs); exit;
					if($rs_medicine_completed->medicine_complete==1 && $rs_medicine_completed->doctor_id==0){
					$i++;
					if($rs_medicine_completed->OldRegistrationNo){
						$registerationNo = $rs_medicine_completed->OldRegistrationNo;
					} else{
						$registerationNo =  'NP';
					}
					if($rs_medicine_completed->shift_id==1){
						$login_shift = 'M';
					} else{
						$login_shift =  'E';
					}
					
					$total_due_amount = $main_total[$key1]['total_due_amount'];
					$recieved_amount = $main_total[$key1]['recieved_amount'];
					$previous_bill_paid_amount = $main_total[$key1]['previous_bill_paid_amount'];
					$visit_due_amount = $main_total[$key1]['visit_due_amount'];
					//$total_due1 = $total_due_amount - $recieved_amount;
					
					if($recieved_amount==$total_due_amount){
                                $remaining_balance_status = '';
                                    $remain_balance = 0.00;
                               }else if($recieved_amount< $total_due_amount){
                                $remaining_balance_status = '+';
                                $balance = $total_due_amount-$recieved_amount;
								$balance_format = number_format((float)$balance, 2, '.', '');
                                $remain_balance = $remaining_balance_status.$balance_format;
                              }else if($recieved_amount > $total_due_amount){
                                $remaining_balance_status = '-';
                                $balance = $recieved_amount-$total_due_amount;
								$balance_format = number_format((float)$balance, 2, '.', '');
                                $remain_balance = $remaining_balance_status.$balance_format;
                              }
					if(count($mode_array[$key1])>0){
						 foreach($mode_array[$key1] as $mode_list){
							   $rm_temp_mode='';
							   foreach($mode_list as $mode_name){
										   $rm_temp_mode .=  $mode_name->transaction_mode.'('.$mode_name->mode_price.')'.", ";
								   }
							   $rmmde['mode_name'] =   rtrim($rm_temp_mode, ", ");
						} 
					}else{
						$rmmde['mode_name'] = 'Mark Complete';
					}
					
					if($previous_due_array[$key1]['transaction_status']=='Debit'){
						$remaining_balance_status='+';
						$balance = $previous_due_array[$key1]['due_amount'];
						$rm_previous_due_balance = $remaining_balance_status.$balance;
					}else if($previous_due_array[$key1]['transaction_status']=='Credit'){
						$remaining_balance_status='-';
						$balance = $previous_due_array[$key1]['due_amount'];
						$rm_previous_due_balance = $balance;
					}else{
						$rm_previous_due_balance =0.00;
					}
					
					//ucwords($rs_medicine_completed->title).' '.
					$objWorkSheet_medicine_completed->SetCellValue('A'.$i, (int)($i - 1));
					$objWorkSheet_medicine_completed->SetCellValue('B'.$i, $registerationNo);
					$objWorkSheet_medicine_completed->SetCellValue('C'.$i, ucwords($rs_medicine_completed->first_name).' '.ucwords($rs_medicine_completed->last_name));
					$objWorkSheet_medicine_completed->SetCellValue('D'.$i, $visit_date);
					$objWorkSheet_medicine_completed->SetCellValue('E'.$i, $rs_medicine_completed->appointment_time);
					$objWorkSheet_medicine_completed->SetCellValue('F'.$i, $login_shift);
					$objWorkSheet_medicine_completed->SetCellValue('G'.$i, $rs_medicine_completed->branch);
					$objWorkSheet_medicine_completed->SetCellValue('H'.$i, $medicine_total[$key1]);
					$objWorkSheet_medicine_completed->SetCellValue('I'.$i, $main_discounts[$key1]['discount_after_med_cost']);
					$objWorkSheet_medicine_completed->SetCellValue('J'.$i, $main_discounts[$key1]['discount_on_med'].'%');
					$objWorkSheet_medicine_completed->SetCellValue('K'.$i, $rm_previous_due_balance);
					$objWorkSheet_medicine_completed->SetCellValue('L'.$i, $charge_array[$key1]);
					$objWorkSheet_medicine_completed->SetCellValue('M'.$i, $previous_bill_paid_amount);
					
					$objWorkSheet_medicine_completed->SetCellValue('N'.$i, $total_due_amount);
					$objWorkSheet_medicine_completed->SetCellValue('O'.$i, $recieved_amount);
					$objWorkSheet_medicine_completed->SetCellValue('P'.$i, $remain_balance);
					$objWorkSheet_medicine_completed->SetCellValue('Q'.$i, 0);
					$objWorkSheet_medicine_completed->SetCellValue('R'.$i, $rmmde['mode_name']);
				 }
				}
				$objWorkSheet_medicine_completed->setTitle('Medicine Completed Queue');
				header('Content-Type: application/vnd.ms-excel');
				header('Content-Disposition: attachment;filename="export_appointments_'.$file_name.'.xlsx"');
				header('Cache-Control: max-age=0');

				$writer = new Xlsx($objPHPExcel);
				//print_r($writer);exit;
				$writer->save('php://output');
		}
}
