<?php
namespace App\Modules\Permissions\Http\Controllers;

use Illuminate\Support\Facades\Validator;

use Illuminate\Http\Request;

use App\Http\Requests;

use Illuminate\Validation\ValidationException;

use App\Http\Controllers\Controller;

use App\Modules\Permissions\Models\Permission;

use Carbon\Carbon;

class PermissionController extends Controller
{
    public function getAll(){
        return Permission::where('status', '1')->where('parent_id', '26')->orderBy('ordering', 'asc')->get();
    }

public function getRecords($id='', Request $request)
    {
        try{
            if($id > 0){

                return Permission::find($id);

            }else if($request->has('query_string')){

                return Permission::where('full_name', 'LIKE', '%' . $request->query_string . '%')->paginate(10);

            }else{

                return Permission::where('status', '!=', '0')->orderBy('id', 'DESC')->paginate(10);

            }
        }catch(\Illuminate\Database\QueryException $ex){

            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);
        }
    }

public function create(Request $request)
    {
        //creating a validator
        $validator = Validator::make($request->all(), [
            'permission_name' => 'required',
            'full_name' => 'required',
            'permission_status' => 'required',
        ]);

        //if validation fails
        if ($validator->fails()) {
            return array(
                'error' => true,
                'message' => $validator->errors()->all()
            );
        }
        try{
                //creating a new permission
                $permission = new Permission();
                //return $request->all();
                //adding values to the permission table
                $permission->permissionName = $request->permission_name;
                $permission->parent_id = $request->permission_category;
                $permission->full_name = $request->full_name;
                $permission->description = $request->permission_description;
                $permission->action_name = $request->menu_alias;
                $permission->icon = $request->menu_icon;
                $permission->is_home = $request->menu_ishome;
                $permission->ordering = $request->menu_ordering;
                $permission->status = $request->permission_status;
                //saving the permission to database
                if($request->id > 0){
                    try{
                        $item = $permission->findOrFail($request->id);
                        if($item){
                            $modification_date =  Carbon::now('Asia/Kolkata');
                            $permission->where('id', $request->id)->update(['action_name'=>$permission->action_name,'icon'=>$permission->icon,'is_home'=>$permission->is_home,'ordering'=>$permission->ordering,'full_name'=>$permission->full_name,'permissionName'=>$permission->permissionName,'parent_id'=>$permission->parent_id,'description'=>$permission->description,'status'=>$permission->status,'modification_date'=>$modification_date]);
                            $res['error'] = false;
                            $res['actions'] = 'update';
                            return response($res, 200);
                        }
                    }catch(\Illuminate\Database\QueryException $ex){
                        $res['status'] = false;
                        $res['message'] = $ex->getMessage();
                        return response($res, 500);

                    }
                }else{

                $permission->save();
                //returning the added permission
                $res['error'] = false;
                $res['actions'] = 'add';
                return response($res, 200);

                }
            }catch(\Illuminate\Database\QueryException $ex){

                $res['status'] = false;
                $res['message'] = $ex->getMessage();
                return response($res, 500);
        }
    }

public function getParentCategory($id=''){
    try{
        if($id > 0){

            return Permission::find($id);

        }else{

          return Permission::where('parent_id', '=', 0)->where('status', '1')->orderBy('id', 'DESC')->get();

        }

       }catch(\Illuminate\Database\QueryException $ex){

            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);

       }
     }

public function delete(Request $request){
    $permission = Permission::findOrFail($request->permission_id);
    try{
        if($permission){

           $permission->where('id', $request->permission_id)->update(['status'=>'0']);

            $res['error'] = false;
            return response($res, 200);

        }else{

         $res['error'] = true;
         return response($res, 401);

        }
    }catch(\Illuminate\Database\QueryException $ex){

        $res['status'] = false;
        $res['message'] = $ex->getMessage();
        return response($res, 500);
     }
    }

     public function categoryWisePermissions(){
        $permission_list = array();
        $categories_data =  Permission::where('parent_id', '0')->where('status', '1')->get(['id','full_name']);
        // clPermissions
        foreach ($categories_data as $category) {

             $permission_data =  Permission::where('parent_id', $category->id)->where('status', '1')->get();
             //return $permission_data;
             $permission_data['cat_id'] = $category->id;

             array_push($permission_list, array("$category->full_name"=>$permission_data));
        }
        return $permission_list;
    }
}
