<?php
namespace App\Modules\User\Http\Controllers;
use Log;

use Illuminate\Support\Facades\Mail;

use App\Mail\SendMail;

use Illuminate\Support\Facades\Hash;

use Illuminate\Support\Facades\Validator;

use Illuminate\Http\Request;

use App\Http\Requests;

use Illuminate\Validation\ValidationException;

use App\Http\Controllers\Controller;

use App\Modules\User\Models\User;

use App\Modules\User\Models\UserBranchMap;

use App\Modules\User\Models\UserHasPermissions;
use Illuminate\Validation\Rule;
use App\Modules\User\Models\UserRole;
use App\Modules\Usertype\Models\Usertype;
use App\Modules\User\Models\DoctorCharges;//This model for doctor charges

use App\Modules\User\Models\DoctorPrints;//This model used for doctor print formats
use App\Modules\Usertype\Models\RoleHasPermissions;
use Carbon\Carbon;
use OwenIt\Auditing\Contracts\Auditable;
class UserController extends Controller
{
    public function getRecords($id='', Request $request)
    {
        try{
            if($id > 0){

				$userDetails = User::find($id);
				$prescriptionFormat = DoctorPrints::select('print_id')->where('user_id', $id)->where('print_type','PRS')->first();
				if($prescriptionFormat){
					$userDetails->presformat = $prescriptionFormat->print_id;
				}
				$receiptFormat= DoctorPrints::select('print_id')->where('user_id', $id)->where('print_type','RCP')->first();
				if($receiptFormat){
					$userDetails->rescpformat = $receiptFormat->print_id;
				}
				return $userDetails;
            }else if($request->has('query_string')){

                return User::where('Name', 'LIKE', '%' . $request->query_string . '%')->where('status','!=','0')->where('user_id', '!=', '1')->paginate(10);

            }else{

                return User::where('status', '!=', '0')->where('user_id', '!=', '1')->orderBy('user_id', 'DESC')->paginate(10);

            }
        }catch(\Illuminate\Database\QueryException $ex){

            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);
        }
    }
   
public function create(Request $request)
    {
			$id = 0;
			if($request->id > 0 ){
				$id = $request->id;
			}
        //creating a validator
        $validator = Validator::make($request->all(), [
            'user_name' => 'required',
            'username' => [
                'required',
                Rule::unique('users')->where(function ($query) use($id) {
                    $query->where('status','!=','0')->where('user_id','!=',$id);
                })
            ],
            'email_id' => 'required',
            'mobile_no' => 'required',
            'password' => 'required',
			'employee_code' => [
                'required',
                Rule::unique('users')->where(function ($query) use($id) {
                    $query->where('status','!=','0')->where('user_id','!=',$id);
                })
            ],
            'user_status' => 'required',
			'label' => [
                'max:5',
                Rule::unique('users')->where(function ($query) use($id) {
                    $query->where('status','!=','0')->where('user_id','!=',$id);
                })
            ],
            'color' =>[
                'max:7',
                Rule::unique('users')->where(function ($query) use($id){
                    $query->where('status','!=','0')->where('color','!=','0')->where('user_id','!=',$id);
                })
            ],
        ]);

        //if validation fails
		//In this below validation return response format we have remove all tag to make in consistent and get error by fields, 10 July 2019, Rahul Khan 
        if ($validator->fails()) {
            return array(
                'error' => true,
                'message' => $validator->errors()
            );
        }
    try{
            $user = new User();
            $branch_map = new UserBranchMap();
            $user_permission = new UserHasPermissions();
            //adding values to the users table
            $hasher = app()->make('hash');
            $user->Name = $request->user_name;
            $user->username = $request->username;
            $user->email = $request->email_id;
            $user->mobile_no = $request->mobile_no;
            $user->password = $hasher->make($request->password);
            $user->status = $request->user_status;
            $user->user_type = $request->user_type;
            $user->label = $request->label;
            $user->ordering = $request->user_order;
            $user->color = $request->color;
			$user->employee_code = $request->employee_code;
          // return $request->doctorCharge;

            if($request->id > 0){
                try{
                $modification_date =  Carbon::now('Asia/Kolkata');

                 // for user branch map delete then create record

                if($request->id> 0 && $request->branches> 0){

                    UserBranchMap::where('user_id', $request->id)->delete();

                    foreach($request->branches as $branch){

                        UserBranchMap::create(['branch_id'=>$branch,'user_id'=>$request->id]);
                    }

                }
				
                 // for user permission delete then create record

                if($request->id> 0 && $request->permissionarray> 0){

                    UserHasPermissions::where('user_id', $request->id)->delete();

                    foreach($request->permissionarray as $permissionindex){
                        $permission = $permissionindex['name'];
                        $permission_value= $permissionindex['value'];
						$permission_name =$permissionindex['permission_name'];
						$route_name =$permissionindex['route_name'];
                            UserHasPermissions::create(['created_by'=>$request->user_id, 'permission_name'=>$permission_name,'route_name'=>$route_name, 'permission_id'=>$permission,'permission_value'=>$permission_value,'user_id'=>$request->id]);
                        
                    }
                }
				

                // for user role delete then create record

                if($request->id> 0 && $request->roles> 0){

                    UserRole::where('user_id', $request->id)->delete();

                    foreach($request->roles as $role){

                        UserRole::create(['role_id' => $role, 'user_id'=>$request->id, 'created_by'=>$request->user_id]);
                    }

                }
				/*These below line is added for saving the data of charges related to doctors, Rahul Khan */
					 if( (isset($request->doctorCharge)) && ($request->doctorCharge > 0)){
						DoctorCharges::where('doctor_id', $request->id)->delete();
						foreach($request->doctorCharge as $doctorCharge){
							DoctorCharges::create(['charge_id' => $doctorCharge['id'], 'doctor_id' => $request->id, 'charge_code' => $doctorCharge['code'], 'price' => $doctorCharge['value'],'print_code' => $doctorCharge['printcode'],'created_by' => $request->user_id]);
						}
					}
					
					
		//This condition is used for insert the Print formats for doctor
				if(($request->user_type==2) and ($request->pres_print > 0)){
					DoctorPrints::where('user_id', $request->id)->where('print_type','PRS')->delete();
					DoctorPrints::create(['user_id' =>$request->id, 'print_id'=>$request->pres_print,'print_type'=>'PRS']);
				}
				
				if(($request->user_type==2) and ($request->rcp_print > 0)){
					DoctorPrints::where('user_id', $request->id)->where('print_type','RCP')->delete();
					DoctorPrints::create(['user_id' =>$request->id, 'print_id'=>$request->rcp_print,'print_type'=>'RCP']);
				}
					
					
                // for user update record
			
                $item = $user->findOrFail($request->id);
                if($item){
                 //'consultancy_fees'=>$user->consultancy_fees,   
                    $user->where('user_id', $request->id)->update(['Name'=>$user->Name,'username'=>$user->username, 'email'=>$user->email, 'mobile_no'=>$user->mobile_no, 'status'=>$user->status, 'user_type'=>$user->user_type, 'label'=>$user->label, 'ordering'=>$user->ordering, 'color'=>$user->color,'employee_code'=>$user->employee_code, 'modification_date'=>$modification_date]);

                    $res['error'] = false;
                    $res['actions'] = 'update';
                    return response($res, 200);
                }
             }catch(\Illuminate\Database\QueryException $ex){

                $res['status'] = false;
                $res['message'] = $ex->getMessage();
                return response($res, 500);

             } 
			 
            }else{
                    $user->save();
                    
                    $user_id =  $user->user_id;
                    
                    $data = array(
                        'username' =>$request->user_name,
                    );

                    
                   // Mail::to($request->email_id)->send(new SendMail($data));
                     // for user branch map create
                     if($user_id && $request->branches> 0){

                        foreach($request->branches as $branch){

                            UserBranchMap::create(['branch_id'=>$branch,'user_id'=>$user_id]);
                            
                        }
                    }

                    // for user role create
                    if($user_id && $request->roles> 0){

                        foreach($request->roles as $role){

                            UserRole::create(['role_id' => $role, 'user_id'=>$user_id, 'created_by'=>$request->user_id]);
                            
                        }
                    }

                    
                    // for user permission create
                   
                    if($user_id && $request->permissionarray> 0){

                            foreach($request->permissionarray as $permissionindex){
                                $permission = $permissionindex['name'];
                                $permission_value= $permissionindex['value'];
								$permission_name =$permissionindex['permission_name'];
								$route_name =$permissionindex['route_name'];
                                    UserHasPermissions::create(['created_by'=>$request->user_id, 'permission_name'=>$permission_name, 'route_name'=>$route_name, 'permission_id'=>$permission,'permission_value'=>$permission_value,'user_id'=>$user_id]);
                                
                            }
                    }
					
					/*These below line is added for saving the data of charges related to doctors, Rahul Khan */
					if( (isset($request->doctorCharge)) && ($request->doctorCharge > 0)){
						//DoctorCharges::where('doctor_id', $request->id)->delete();
						foreach($request->doctorCharge as $doctorCharge){
							DoctorCharges::create(['charge_id' => $doctorCharge['id'], 'print_code' => $doctorCharge['printcode'], 'doctor_id' =>$user_id , 'charge_code' => $doctorCharge['code'], 'price' => $doctorCharge['value']]);
						}
					}
	//This condition is used for insert the Print formats for doctor
				if(($request->user_type==2) and ($request->pres_print > 0)){
					DoctorPrints::create(['user_id' =>$user_id, 'print_id'=>$request->pres_print,'print_type'=>'PRS']);
				}
				
				if(($request->user_type==2) and ($request->rcp_print > 0)){
					DoctorPrints::create(['user_id' =>$user_id, 'print_id'=>$request->rcp_print,'print_type'=>'RCP']);
				}
			

            $res['error'] = false;
            $res['actions'] = 'add';
            return response($res, 200);

          }
        }catch(\Illuminate\Database\QueryException $ex){

            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);
        }
    }

public function getParentCategory($id=''){
    try{
        if($id > 0){

            return User::find($id);

        }else{

          return User::where('parent_id', '=', 0)->orderBy('id', 'DESC')->get();

        }

       }catch(\Illuminate\Database\QueryException $ex){

            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);

       }
     }

public function delete(Request $request){
    $user = User::findOrFail($request->user_id);
    try{
        if($user){

           $user->where('user_id', $request->user_id)->update(['status'=>'0']);

            $res['error'] = false;
            return response($res, 200);

        }else{

         $res['error'] = true;
         return response($res, 401);

        }
    }catch(\Illuminate\Database\QueryException $ex){

        $res['status'] = false;
        $res['message'] = $ex->getMessage();
        return response($res, 500);
     }
    }

     public function authenticate(Request $request)

      {
        $this->validate($request, [

            'username' => 'required',

            'password' => 'required'

        ]);
        $location =  $request->location_id;

        $location_array = explode("-", $location);

        $user = User::where('username', $request->username)->where('status', '1')->first();
		//return $user;

        if(!empty($user)){

            //Log::info('User Password='.$user->password);

            //Log::info('User Name='.$user->username); 
    
            //Log::info('Branch Name='.$location_array[0]); 

                $user_permissions_array = array();
				$user_route_name_list_array = array();
				$role_permissions_array = array();
				$route_role_permissions_array = array();
    //This below line code is used for setting the flag of reset password, Rahul Khan 31 July 2019
			//	$modification_date =  Carbon::now('Asia/Kolkata');
				User::where('user_id', $user->user_id)->update(['reset'=>'0']);
				
                $permissions_list = UserHasPermissions::select('permission_name')->where('user_id', $user->user_id)->where('permission_value', 'Allow')->get();
                //$permissions_list = UserHasPermissions::find($user->user_id)->getUserMultiplePermissions;
                foreach($permissions_list as $list){
                    $user_permissions_array[] =$list->permission_name;

                    //array_push($permissions_array, $list);
                }
			   $route_name_list = UserHasPermissions::select('route_name')->where('user_id', $user->user_id)->where('permission_value', 'Allow')->get();

                foreach($route_name_list as $list){
                    $user_route_name_list_array[] =$list->route_name;

                    //array_push($permissions_array, $list);
                }
				
				
			 $multiple_role_id= $this->getUserRole($user->user_id);
			 foreach($multiple_role_id as $role_id){
				 
				$role_permissions_list = RoleHasPermissions::select('permission_name')->where('role_id', $role_id)->groupBy('permission_name')->get();
                //$permissions_list = UserHasPermissions::find($user->user_id)->getUserMultiplePermissions;
				
				
                foreach($role_permissions_list as $list){
                    $role_permissions_array[] =$list->permission_name;

                    //array_push($permissions_array, $list);
                } 
				 
			 }
			 
			$multiple_role_id= $this->getUserRole($user->user_id);
			 foreach($multiple_role_id as $role_id){
				$role_permissions_list = RoleHasPermissions::select('route_name')->where('role_id', $role_id)->groupBy('route_name')->get();
                //$permissions_list = UserHasPermissions::find($user->user_id)->getUserMultiplePermissions;
                foreach($role_permissions_list as $list){
					if($list->route_name!=''){
                       $route_role_permissions_array[] =$list->route_name;
					}

                    //array_push($permissions_array, $list);
                } 
				 
			 }
			 
			 //dd($this->my_array_unique($role_permissions_array));
            $branch = UserBranchMap::where('branch_id', $location_array[0])->where('user_id', $user->user_id)->first();
            if(!empty($branch)){

                if(Hash::check($request->input('password'), $user->password)){
                $apikey = base64_encode(str_random(40));
                User::where('username', $request->input('username'))->update(['remember_token' => "$apikey"]);
                $res['validate'] =true;
                $res['token'] = $apikey;
                $res['user_id'] = $user->user_id;
                $res['name'] = $user->Name;
                $res['user_type'] = $user->user_type;
                $res['label'] = $user->label;
				$res['printed_by'] = $user->employee_code;
                $res['user_permission'] = $this->my_array_unique($role_permissions_array);
				//$res['user_route'] = $user_route_name_list_array;
				//$res['role_permission'] = $user_permissions_array;
				$res['role_route'] = $this->my_array_unique($route_role_permissions_array);
                return response($res, 200);
                }else if($user->reset==1){
						$res['validate'] = 'passwordreset';
                        return response($res, 200);
				}else{
                        $res['validate'] = 'invalid';
                        return response($res, 200);
                    }
                }else{

                    $res['validate'] = 'nobranch';
                    $res['name'] =$location_array[1];
                    return response($res, 200);
                            
                }

      }else{

                    $res['validate'] = 'nouser';
                    return response($res, 200);
           }

   }
   
   // This function used for removing dublicate role permission name
	 function my_array_unique($array, $keep_key_assoc = false){
		$duplicate_keys = array();
		$tmp = array();       

		foreach ($array as $key => $val){
			// convert objects to arrays, in_array() does not support objects
			if (is_object($val))
				$val = (array)$val;

			if (!in_array($val, $tmp))
				$tmp[] = $val;
			else
				$duplicate_keys[] = $key;
		}

		foreach ($duplicate_keys as $key)
			unset($array[$key]);

		return $keep_key_assoc ? $array : array_values($array);
	}
    public function getUserBranchMap($id){

        $branch_array = array();
        $branch_list = User::find($id)->getMultipleBranch;

        foreach($branch_list as $list){

            array_push($branch_array, $list->branch_id);
        }
        return $branch_array;
    }

    public function getUserHasPermissions($id){

        $permissions_array = array();

        $permissions_list = User::find($id)->getUserMultiplePermissions;

        foreach($permissions_list as $list){

            array_push($permissions_array, ['permission'=>$list->permission_id, 'value'=>$list->permission_value]);
        }
        return $permissions_array;
    }

    public function getUserRole($id){

        $userrole_array = array();

        $userrole_list = User::find($id)->getUserMultipleRole;

        foreach($userrole_list as $list){
            //This below code is used for getting the only active user type from the user profile, Rahul Khan 02 Aug 2019
            $roleTypeid = Usertype::select('id')->where('id', $list->role_id)->where('status','1')->get()->first();

            if((isset($roleTypeid->id)) and ($roleTypeid->id > 0)){
				array_push($userrole_array, $list->role_id);
			}
           
        }
        return $userrole_array;
    }
	/*This below function is used for getting doctors' charges price, Rahul Khan*/
	public function getDoctorCharges($id){
			return DoctorCharges::with('charge')->where('doctor_id',$id)->get();
	}
	/*This below function is used for getting the resetpassword */
	public function resetpassword(Request $request){
	
		$user = User::where('username', $request->username)->get()->first();
		//return $user;
		if (!$user) {
			$error_message = "Your username was not found.";
			return response()->json(['success' => false, 'error' => ['username'=> $error_message]], 200);
		}
		try {
			$chars = "abcdefghijklmnopqrstuvwxyz0123456789";
			$genaratedPass = substr(str_shuffle($chars),0,8);
			$hasher = app()->make('hash');
			$hashpassword = $hasher->make($genaratedPass);
			//return $hashpassword;
			$modification_date =  Carbon::now('Asia/Kolkata');
			User::where('user_id', $user->user_id)->update(['password'=>$hashpassword,'reset'=>'1','modification_date'=>$modification_date]);
			$data = array();
			$data['name'] = $user->Name;
			$data['username'] = $user->username;
			$data['password'] = $genaratedPass;
			
			
			Mail::to('shubh.tiwari@computerware.co')->send(new SendMail($data));

		} catch (\Exception $e) {
			$error_message = $e->getMessage();
			return response()->json(['error' => true, 'error' => $error_message], 401);
		}
		return response()->json([
			'success' => true, 'data'=> ['message'=> 'A reset email has been sent! Please check your email.']
		]);
		
    }
//This below function is used for updating the user profile
    public function updateProfile(Request $request){
        //creating a validator
        $validator = Validator::make($request->all(), [
            'user_name' => 'required',
            'email_id' => 'required',
            'mobile_no' => 'required',
        ]);
        if ($validator->fails()) {
            return array(
                'error' => true,
                'message' => $validator->errors()
            );
        }
        try{
             $user = new User();
             $modification_date =  Carbon::now('Asia/Kolkata');
             //adding values to the users table
            $hasher = app()->make('hash');
            $user->Name = $request->user_name;
            $user->email = $request->email_id;
            $user->mobile_no = $request->mobile_no;
            $user->password = $hasher->make($request->password);
            //return $request->password;
            if($request->password=='0'){
                 $data = [
                        'Name'=>$user->Name,
                        'email'=>$user->email,
                        'mobile_no'=>$user->mobile_no,
                        'modification_date'=>$modification_date,
                 ];
            }else{
                $data = [
                    'Name'=>$user->Name,
                    'email'=>$user->email,
                    'mobile_no'=>$user->mobile_no,
                    'password'=> $user->password,
                    'modification_date'=>$modification_date,
                ];
             }
            // return $data;
            $user->where('user_id', $request->id)->update($data);
                $res['error'] = false;
                $res['actions'] = 'update';
                return response($res, 200);
          
        }catch(\Illuminate\Database\QueryException $ex){

            $res['status'] = false;
            $res['message'] = $ex->getMessage();
            return response($res, 500);
        }
    }
	public function logoutUser(Request $request){
       $user = User::where('user_id', $request->u)->where('status', '1')->first();
        if(!empty($user)){
            if(Hash::check($request->p, $user->password)){
                $res['validate'] = true;
                return response($res, 200);
            }else{
                $res['validate'] = false;
                return response($res, 200);
            }
        }
    }
}

